@@ -17,15 +17,16 @@ function get_sources(rockspec, extract, dest_dir)
    assert(type(rockspec) == "table")
    assert(type(dest_dir) == "string" or not dest_dir)
 
+   local git_cmd = rockspec.variables.GIT
    local name_version = rockspec.name .. "-" .. rockspec.version
    local module = dir.base_name(rockspec.source.url)
    -- Strip off .git from base name if present
    module = module:gsub("%.git$", "")
-   local command = {"git", "clone", rockspec.source.url, module}
+   local command = {git_cmd, "clone", "--depth=1", rockspec.source.url, module}
    local checkout_command
    local tag_or_branch = rockspec.source.tag or rockspec.source.branch
    if tag_or_branch then
-      checkout_command = {"git", "checkout", tag_or_branch}
+      checkout_command = {git_cmd, "checkout", "-q", tag_or_branch}
    end
    local store_dir
    if not dest_dir then
@@ -37,17 +38,20 @@ function get_sources(rockspec, extract, dest_dir)
    else
       store_dir = dest_dir
    end
+   store_dir = fs.absolute_name(store_dir)
    fs.change_dir(store_dir)
    if not fs.execute(unpack(command)) then
-      return nil, "Failed fetching files from GIT while cloning."
+      return nil, "Failed cloning git repository."
    end
+   fs.change_dir(module)
    if checkout_command then
-      fs.change_dir(module)
       if not fs.execute(unpack(checkout_command)) then
-         return nil, "Failed fetching files from GIT while getting tag/branch."
+         return nil, "Failed checking out tag/branch from git repository."
       end
-      fs.pop_dir()
    end
+   fs.delete(dir.path(store_dir, module, ".git"))
+   fs.delete(dir.path(store_dir, module, ".gitignore"))
+   fs.pop_dir()
    fs.pop_dir()
    return module, store_dir
 end
