@@ -8,13 +8,13 @@ local fs = require("luarocks.fs")
 local cfg = require("luarocks.cfg")
 local dir = require("luarocks.dir")
 local util = require("luarocks.util")
+local path = require("luarocks.path")
 
 local socket_ok, http = pcall(require, "socket.http")
 local _, ftp = pcall(require, "socket.ftp")
 local zip_ok, lrzip = pcall(require, "luarocks.tools.zip")
 local unzip_ok, luazip = pcall(require, "zip"); _G.zip = nil
 local lfs_ok, lfs = pcall(require, "lfs")
---local curl_ok, curl = pcall(require, "luacurl")
 local md5_ok, md5 = pcall(require, "md5")
 local posix_ok, posix = pcall(require, "posix")
 
@@ -38,6 +38,10 @@ function Q(arg)
    return "'" .. arg:gsub("\\", "\\\\"):gsub("'", "'\\''") .. "'"
 end
 
+local function normalize(name)
+   return name:gsub("\\", "/"):gsub("/$", "")
+end
+
 --- Test is file/dir is writable.
 -- Warning: testing if a file/dir is writable does not guarantee
 -- that it will remain writable and therefore it is no replacement
@@ -46,9 +50,10 @@ end
 -- @return boolean: true if file exists, false otherwise.
 function is_writable(file)
    assert(file)
+   file = normalize(file)
    local result
    if fs.is_dir(file) then
-      local file2 = file .. '/.tmpluarockstestwritable'
+      local file2 = dir.path(file, '.tmpluarockstestwritable')
       local fh = io.open(file2, 'wb')
       result = fh ~= nil
       if fh then fh:close() end
@@ -67,8 +72,8 @@ end
 -- @return string or nil: name of temporary directory or nil on failure.
 function make_temp_dir(name)
    assert(type(name) == "string")
+   name = normalize(name)
 
-   name = name:gsub("\\", "/")
    local temp_dir = (os.getenv("TMP") or "/tmp") .. "/luarocks_" .. name:gsub(dir.separator, "_") .. "-" .. tostring(math.floor(math.random() * 10000))
    if fs.make_dir(temp_dir) then
       return temp_dir
@@ -100,6 +105,7 @@ end
 -- @return boolean: true if the MD5 checksum for 'file' equals 'md5sum', false if not
 -- or if it could not perform the check for any reason.
 function check_md5(file, md5sum)
+   file = normalize(file)
    local computed = fs.get_md5(file)
    if not computed then
       return false
@@ -144,6 +150,7 @@ end
 -- @param d string: The directory to switch to.
 function change_dir(d)
    table.insert(dir_stack, lfs.currentdir())
+   d = normalize(d)
    lfs.chdir(d)
 end
 
@@ -152,8 +159,7 @@ end
 -- a crossplatform way.
 function change_dir_to_root()
    table.insert(dir_stack, lfs.currentdir())
-   -- TODO Does this work on Windows?
-   lfs.chdir("/")
+   lfs.chdir("/") -- works on Windows too
 end
 
 --- Change working directory to the previous in the dir stack.
@@ -175,7 +181,7 @@ end
 -- @return boolean: true on success, false on failure.
 function make_dir(directory)
    assert(type(directory) == "string")
-   directory = directory:gsub("\\", "/")
+   directory = normalize(directory)
    local path = nil
    if directory:sub(2, 2) == ":" then
      path = directory:sub(1, 2)
@@ -205,6 +211,7 @@ end
 -- @param d string: pathname of directory to remove.
 function remove_dir_if_empty(d)
    assert(d)
+   d = normalize(d)
    lfs.rmdir(d)
 end
 
@@ -214,6 +221,7 @@ end
 -- @param d string: pathname of directory to remove.
 function remove_dir_tree_if_empty(d)
    assert(d)
+   d = normalize(d)
    for i=1,10 do
       lfs.rmdir(d)
       d = dir.dir_name(d)
@@ -223,14 +231,19 @@ end
 --- Copy a file.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
+-- @param perms string or nil: Permissions for destination file,
+-- or nil to use the source filename permissions
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
-function copy(src, dest)
+function copy(src, dest, perms)
    assert(src and dest)
+   src = normalize(src)
+   dest = normalize(dest)
    local destmode = lfs.attributes(dest, "mode")
    if destmode == "directory" then
       dest = dir.path(dest, dir.base_name(src))
    end
+   if not perms then perms = fs.get_permissions(src) end
    local src_h, err = io.open(src, "rb")
    if not src_h then return nil, err end
    local dest_h, err = io.open(dest, "wb+")
@@ -242,10 +255,12 @@ function copy(src, dest)
    end
    src_h:close()
    dest_h:close()
+   fs.chmod(dest, perms)
    return true
 end
 
 --- Implementation function for recursive copy of directory contents.
+-- Assumes paths are normalized.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
 -- @return boolean or (boolean, string): true on success, false on failure
@@ -275,6 +290,8 @@ end
 -- plus an error message.
 function copy_contents(src, dest)
    assert(src and dest)
+   src = normalize(src)
+   dest = normalize(dest)
    assert(lfs.attributes(src, "mode") == "directory")
 
    for file in lfs.dir(src) do
@@ -289,35 +306,34 @@ function copy_contents(src, dest)
 end
 
 --- Implementation function for recursive removal of directories.
--- @param src string: Pathname of source
--- @param dest string: Pathname of destination
+-- Assumes paths are normalized.
+-- @param name string: Pathname of file
 -- @return boolean or (boolean, string): true on success,
 -- or nil and an error message on failure.
-local function recursive_delete(src)
-   local srcmode = lfs.attributes(src, "mode")
+local function recursive_delete(name)
+   local mode = lfs.attributes(name, "mode")
 
-   if srcmode == "file" then
-      return os.remove(src)
-   elseif srcmode == "directory" then
-      for file in lfs.dir(src) do
+   if mode == "file" then
+      return os.remove(name)
+   elseif mode == "directory" then
+      for file in lfs.dir(name) do
          if file ~= "." and file ~= ".." then
-            local ok, err = recursive_delete(dir.path(src, file))
+            local ok, err = recursive_delete(dir.path(name, file))
             if not ok then return nil, err end
          end
       end
-      local ok, err = lfs.rmdir(src)
+      local ok, err = lfs.rmdir(name)
       if not ok then return nil, err end
    end
    return true
 end
 
 --- Delete a file or a directory and all its contents.
--- For safety, this only accepts absolute paths.
--- @param arg string: Pathname of source
+-- @param name string: Pathname of source
 -- @return boolean: true on success, false on failure.
-function delete(arg)
-   assert(arg)
-   return recursive_delete(arg) or false
+function delete(name)
+   name = normalize(name)
+   return recursive_delete(name) or false
 end
 
 --- List the contents of a directory.
@@ -330,6 +346,7 @@ function list_dir(at)
    if not at then
       at = fs.current_dir()
    end
+   at = normalize(at)
    if not fs.is_dir(at) then
       return {}
    end
@@ -343,6 +360,7 @@ function list_dir(at)
 end
 
 --- Implementation function for recursive find.
+-- Assumes paths are normalized.
 -- @param cwd string: Current working directory in recursion.
 -- @param prefix string: Auxiliary prefix string to form pathname.
 -- @param result table: Array of strings where results are collected.
@@ -369,6 +387,7 @@ function find(at)
    if not at then
       at = fs.current_dir()
    end
+   at = normalize(at)
    if not fs.is_dir(at) then
       return {}
    end
@@ -382,6 +401,7 @@ end
 -- @return boolean: true if file exists, false otherwise.
 function exists(file)
    assert(file)
+   file = normalize(file)
    return type(lfs.attributes(file)) == "table"
 end
 
@@ -390,6 +410,7 @@ end
 -- @return boolean: true if it is a directory, false otherwise.
 function is_dir(file)
    assert(file)
+   file = normalize(file)
    return lfs.attributes(file, "mode") == "directory"
 end
 
@@ -398,10 +419,12 @@ end
 -- @return boolean: true if it is a file, false otherwise.
 function is_file(file)
    assert(file)
+   file = normalize(file)
    return lfs.attributes(file, "mode") == "file"
 end
 
 function set_time(file, time)
+   file = normalize(file)
    return lfs.touch(file, time)
 end
 
@@ -424,68 +447,27 @@ if unzip_ok then
 -- @param zipfile string: pathname of .zip archive to be extracted.
 -- @return boolean: true on success, false on failure.
 function unzip(zipfile)
-  local zipfile, err = luazip.open(zipfile)
-  if not zipfile then return nil, err end
-  local files = zipfile:files()
-  local file = files()
-  repeat
-	if file.filename:sub(#file.filename) == "/" then
-	  fs.make_dir(dir.path(fs.current_dir(), file.filename))
-	else
-      local rf, err = zipfile:open(file.filename)
-	  if not rf then zipfile:close(); return nil, err end
-	  local contents = rf:read("*a")
-	  rf:close()
-	  local wf, err = io.open(dir.path(fs.current_dir(), file.filename), "wb")
-	  if not wf then zipfile:close(); return nil, err end
-	  wf:write(contents)
-	  wf:close()
-	end
-	file = files()
-  until not file
-  zipfile:close()
-  return true
-end
-
-end
-
----------------------------------------------------------------------
--- LuaCurl functions
----------------------------------------------------------------------
-
-if curl_ok then
-
---- Download a remote file.
--- @param url string: URL to be fetched.
--- @param filename string or nil: this function attempts to detect the
--- resulting local filename of the remote file as the basename of the URL;
--- if that is not correct (due to a redirection, for example), the local
--- filename can be given explicitly as this second argument.
--- @return boolean: true on success, false on failure.
-function download(url, filename)
-   assert(type(url) == "string")
-   assert(type(filename) == "string" or not filename)
-
-   filename = dir.path(fs.current_dir(), filename or dir.base_name(url))
-
-   local c = curl.new()
-   if not c then return false end
-   local file = io.open(filename, "wb")
-   if not file then return false end
-   local ok = c:setopt(curl.OPT_WRITEFUNCTION, function (stream, buffer)
-      stream:write(buffer)
-      return string.len(buffer)
-   end)
-   ok = ok and c:setopt(curl.OPT_WRITEDATA, file)
-   ok = ok and c:setopt(curl.OPT_BUFFERSIZE, 5000)
-   ok = ok and c:setopt(curl.OPT_HTTPHEADER, "Connection: Keep-Alive")
-   ok = ok and c:setopt(curl.OPT_URL, url)
-   ok = ok and c:setopt(curl.OPT_CONNECTTIMEOUT, 15)
-   ok = ok and c:setopt(curl.OPT_USERAGENT, cfg.user_agent)
-   ok = ok and c:perform()
-   ok = ok and c:close()
-   file:close()
-   return ok
+   local zipfile, err = luazip.open(zipfile)
+   if not zipfile then return nil, err end
+   local files = zipfile:files()
+   local file = files()
+   repeat
+      if file.filename:sub(#file.filename) == "/" then
+         fs.make_dir(dir.path(fs.current_dir(), file.filename))
+      else
+         local rf, err = zipfile:open(file.filename)
+         if not rf then zipfile:close(); return nil, err end
+         local contents = rf:read("*a")
+         rf:close()
+         local wf, err = io.open(dir.path(fs.current_dir(), file.filename), "wb")
+         if not wf then zipfile:close(); return nil, err end
+         wf:write(contents)
+         wf:close()
+      end
+      file = files()
+   until not file
+   zipfile:close()
+   return true
 end
 
 end
@@ -496,6 +478,42 @@ end
 
 if socket_ok then
 
+local ltn12 = require("ltn12")
+local luasec_ok, https = pcall(require, "ssl.https")
+local redirect_protocols = {
+   http = http,
+   https = luasec_ok and https,
+}
+
+local function http_request(url, http, loop_control)
+   local result = {}
+   local res, status, headers, err = http.request { url = url, proxy = cfg.proxy, redirect = false, sink = ltn12.sink.table(result) }
+   if not res then
+      return nil, status
+   elseif status == 301 or status == 302 then
+      local location = headers.location
+      if location then
+         local protocol, rest = dir.split_url(location)
+         if redirect_protocols[protocol] then
+            if not loop_control then
+               loop_control = {}
+            elseif loop_control[location] then
+               return nil, "Redirection loop -- broken URL?"
+            end
+            loop_control[url] = true
+            return http_request(location, redirect_protocols[protocol], loop_control)
+         else
+            return nil, "URL redirected to unsupported protocol - install luasec to get HTTPS support."
+         end
+      end
+      return nil, err
+   elseif status ~= 200 then
+      return nil, err
+   else
+      return table.concat(result)
+   end
+end
+
 --- Download a remote file.
 -- @param url string: URL to be fetched.
 -- @param filename string or nil: this function attempts to detect the
@@ -508,22 +526,23 @@ function download(url, filename)
    assert(type(filename) == "string" or not filename)
 
    filename = dir.path(fs.current_dir(), filename or dir.base_name(url))
-
+   
    local content, err
    if util.starts_with(url, "http:") then
-      local res, status, headers, line = http.request(url)
-      if not res then
-         err = status
-      elseif status ~= 200 then
-         err = line
-      else
-         content = res
-      end
+      content, err = http_request(url, http)
    elseif util.starts_with(url, "ftp:") then
       content, err = ftp.get(url)
+   elseif util.starts_with(url, "https:") then
+      if luasec_ok then
+         content, err = http_request(url, https)
+      else
+         err = "Unsupported protocol - install luasec to get HTTPS support."
+      end
+   else
+      err = "Unsupported protocol"
    end
    if not content then
-      return false, "Failed downloading: " .. err
+      return false, tostring(err)
    end
    local file = io.open(filename, "wb")
    if not file then return false end
@@ -533,7 +552,6 @@ function download(url, filename)
 end
 
 end
-
 ---------------------------------------------------------------------
 -- MD5 functions
 ---------------------------------------------------------------------
@@ -560,11 +578,34 @@ end
 
 if posix_ok then
 
+local octal_to_rwx = {
+   ["0"] = "---",
+   ["1"] = "--x",
+   ["2"] = "-w-",
+   ["3"] = "-wx",
+   ["4"] = "r--",
+   ["5"] = "r-x",
+   ["6"] = "rw-",
+   ["7"] = "rwx",
+}
+
 function chmod(file, mode)
+   -- LuaPosix (as of 5.1.15) does not support octal notation...
+   if mode:sub(1,1) == "0" then
+      local new_mode = {}
+      for c in mode:sub(2):gmatch(".") do
+         table.insert(new_mode, octal_to_rwx[c])
+      end
+      mode = table.concat(new_mode)
+   end
    local err = posix.chmod(file, mode)
    return err == 0
 end
 
+function get_permissions(file)
+   return posix.stat(file, "mode")
+end
+
 end
 
 ---------------------------------------------------------------------
@@ -573,6 +614,7 @@ end
 
 --- Apply a patch.
 -- @param patchname string: The filename of the patch.
+-- @param patchdata string or nil: The actual patch as a string.
 function apply_patch(patchname, patchdata)
    local p, all_ok = patch.read_patch(patchname, patchdata)
    if not all_ok then
@@ -603,3 +645,17 @@ function move(src, dest)
    end
    return true
 end
+
+--- Check if user has write permissions for the command.
+-- Assumes the configuration variables under cfg have been previously set up.
+-- @param flags table: the flags table passed to run() drivers.
+-- @return boolean or (boolean, string): true on success, false on failure,
+-- plus an error message.
+function check_command_permissions(flags)
+   local root_dir = path.root_dir(cfg.rocks_dir)
+   if not flags["local"] and not (fs.is_writable(root_dir) or fs.is_writable(dir.dir_name(root_dir))) then
+      return nil, "Your user does not have write permissions in " .. root_dir ..
+                  " \n-- you may want to run as a privileged user or use your local tree with --local."
+   end
+   return true
+end
