@@ -8,8 +8,10 @@ local cfg = require("luarocks.cfg")
 
 local dir_stack = {}
 
+local vars = cfg.variables
+
 --- Run the given command.
--- The command is executed in the current directory in the dir stack.
+-- The command is executed in the current directory in the directory stack.
 -- @param cmd string: No quoting/escaping is applied to the command.
 -- @return boolean: true if command succeeds (status code 0), false
 -- otherwise.
@@ -23,29 +25,26 @@ function execute_string(cmd)
 end
 
 --- Obtain current directory.
--- Uses the module's internal dir stack.
+-- Uses the module's internal directory stack.
 -- @return string: the absolute pathname of the current directory.
 function current_dir()
-   local current = os.getenv("PWD")
-   if not current then
-      local pipe = io.popen("pwd")
-      current = pipe:read("*l")
-      pipe:close()
-   end
-   for _, d in ipairs(dir_stack) do
-      current = fs.absolute_name(d, current)
+   local pipe = io.popen(vars.PWD)
+   local current = pipe:read("*l")
+   pipe:close()
+   for _, directory in ipairs(dir_stack) do
+      current = fs.absolute_name(directory, current)
    end
    return current
 end
 
 --- Change the current directory.
--- Uses the module's internal dir stack. This does not have exact
+-- Uses the module's internal directory stack. This does not have exact
 -- semantics of chdir, as it does not handle errors the same way,
 -- but works well for our purposes for now.
--- @param d string: The directory to switch to.
-function change_dir(d)
-   assert(type(d) == "string")
-   table.insert(dir_stack, d)
+-- @param directory string: The directory to switch to.
+function change_dir(directory)
+   assert(type(directory) == "string")
+   table.insert(dir_stack, directory)
 end
 
 --- Change directory to root.
@@ -55,48 +54,59 @@ function change_dir_to_root()
    table.insert(dir_stack, "/")
 end
 
---- Change working directory to the previous in the dir stack.
+--- Change working directory to the previous in the directory stack.
 function pop_dir()
-   local d = table.remove(dir_stack)
-   return d ~= nil
+   local directory = table.remove(dir_stack)
+   return directory ~= nil
 end
 
 --- Create a directory if it does not already exist.
 -- If any of the higher levels in the path name does not exist
 -- too, they are created as well.
--- @param d string: pathname of directory to create.
+-- @param directory string: pathname of directory to create.
 -- @return boolean: true on success, false on failure.
-function make_dir(d)
-   assert(d)
-   return fs.execute("mkdir -p", d)
+function make_dir(directory)
+   assert(directory)
+   return fs.execute(vars.MKDIR.." -p", directory)
 end
 
 --- Remove a directory if it is empty.
 -- Does not return errors (for example, if directory is not empty or
 -- if already does not exist)
--- @param dir string: pathname of directory to remove.
-function remove_dir_if_empty(d)
-   assert(d)
-   fs.execute_string("rmdir "..fs.Q(d).." 1> /dev/null 2> /dev/null")
+-- @param directory string: pathname of directory to remove.
+function remove_dir_if_empty(directory)
+   assert(directory)
+   fs.execute_string(vars.RMDIR.." "..fs.Q(directory).." 1> /dev/null 2> /dev/null")
 end
 
 --- Remove a directory if it is empty.
 -- Does not return errors (for example, if directory is not empty or
 -- if already does not exist)
--- @param dir string: pathname of directory to remove.
-function remove_dir_tree_if_empty(d)
-   assert(d)
-   fs.execute_string("rmdir -p "..fs.Q(d).." 1> /dev/null 2> /dev/null")
+-- @param directory string: pathname of directory to remove.
+function remove_dir_tree_if_empty(directory)
+   assert(directory)
+   fs.execute_string(vars.RMDIR.." -p "..fs.Q(directory).." 1> /dev/null 2> /dev/null")
 end
 
 --- Copy a file.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
+-- @param perm string or nil: Permissions for destination file,
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
-function copy(src, dest)
+function copy(src, dest, perm)
    assert(src and dest)
-   if fs.execute("cp", src, dest) then
+   if fs.execute(vars.CP, src, dest) then
+      if perm then
+         if fs.is_dir(dest) then
+            dest = dir.path(dest, dir.base_name(src))
+         end
+         if fs.chmod(dest, perm) then
+            return true
+         else
+            return false, "Failed setting permissions of "..dest
+         end
+      end
       return true
    else
       return false, "Failed copying "..src.." to "..dest
@@ -110,7 +120,7 @@ end
 -- plus an error message.
 function copy_contents(src, dest)
    assert(src and dest)
-   if fs.execute_string("cp -pPR "..fs.Q(src).."/* "..fs.Q(dest).." 1> /dev/null 2>/dev/null") then
+   if fs.execute_string(vars.CP.." -pPR "..fs.Q(src).."/* "..fs.Q(dest).." 1> /dev/null 2>/dev/null") then
       return true
    else
       return false, "Failed copying "..src.." to "..dest
@@ -123,7 +133,7 @@ end
 function delete(arg)
    assert(arg)
    assert(arg:sub(1,1) == "/")
-   return fs.execute_string("rm -rf " .. fs.Q(arg) .. " 1> /dev/null 2>/dev/null")
+   return fs.execute_string(vars.RM.." -rf " .. fs.Q(arg) .. " 1> /dev/null 2>/dev/null")
 end
 
 --- List the contents of a directory. 
@@ -140,7 +150,7 @@ function list_dir(at)
       return {}
    end
    local result = {}
-   local pipe = io.popen("cd "..fs.Q(at).." && ls")
+   local pipe = io.popen("cd "..fs.Q(at).." && "..vars.LS)
    for file in pipe:lines() do
       table.insert(result, file)
    end
@@ -162,7 +172,7 @@ function find(at)
       return {}
    end
    local result = {}
-   local pipe = io.popen("cd "..fs.Q(at).." && find * 2>/dev/null") 
+   local pipe = io.popen("cd "..fs.Q(at).." && "..vars.FIND.." * 2>/dev/null") 
    for file in pipe:lines() do
       table.insert(result, file)
    end
@@ -176,7 +186,7 @@ end
 -- additional arguments.
 -- @return boolean: true on success, false on failure.
 function zip(zipfile, ...)
-   return fs.execute("zip -r", zipfile, ...)
+   return fs.execute(vars.ZIP.." -r", zipfile, ...)
 end
 
 --- Uncompress files from a .zip archive.
@@ -184,7 +194,7 @@ end
 -- @return boolean: true on success, false on failure.
 function unzip(zipfile)
    assert(zipfile)
-   return fs.execute("unzip", zipfile)
+   return fs.execute(vars.UNZIP, zipfile)
 end
 
 --- Test for existance of a file.
@@ -192,15 +202,15 @@ end
 -- @return boolean: true if file exists, false otherwise.
 function exists(file)
    assert(file)
-   return fs.execute("test -r", file)
+   return fs.execute(vars.TEST, "-r", file)
 end
 
---- Test is file/dir is writable.
+--- Test is file/directory is writable.
 -- @param file string: filename to test
 -- @return boolean: true if file exists, false otherwise.
 function is_writable(file)
    assert(file)
-   return fs.execute("test -w", file)
+   return fs.execute(vars.TEST, "-w", file)
 end
 
 --- Test is pathname is a directory.
@@ -208,7 +218,7 @@ end
 -- @return boolean: true if it is a directory, false otherwise.
 function is_dir(file)
    assert(file)
-   return fs.execute("test -d", file)
+   return fs.execute(vars.TEST, "-d", file)
 end
 
 --- Test is pathname is a regular file.
@@ -216,7 +226,7 @@ end
 -- @return boolean: true if it is a regular file, false otherwise.
 function is_file(file)
    assert(file)
-   return fs.execute("test -f", file)
+   return fs.execute(vars.TEST, "-f", file)
 end
 
 --- Download a remote file.
@@ -231,7 +241,7 @@ function download(url, filename)
    assert(type(filename) == "string" or not filename)
 
    if cfg.downloader == "wget" then
-      local wget_cmd = "wget --no-cache --user-agent="..cfg.user_agent.." --quiet --continue "
+      local wget_cmd = vars.WGET.." --no-check-certificate --no-cache --user-agent="..cfg.user_agent.." --quiet --continue "
       if filename then
          return fs.execute(wget_cmd.." --output-document ", filename, url)
       else
@@ -239,18 +249,22 @@ function download(url, filename)
       end
    elseif cfg.downloader == "curl" then
       filename = filename or dir.base_name(url)
-      return fs.execute_string("curl --user-agent "..cfg.user_agent.." "..fs.Q(url).." 2> /dev/null 1> "..fs.Q(filename))
+      return fs.execute_string(vars.CURL.." -L --user-agent "..cfg.user_agent.." "..fs.Q(url).." 2> /dev/null 1> "..fs.Q(filename))
    end
 end
 
 function chmod(pathname, mode)
-  return fs.execute("chmod "..mode, pathname)
+   if mode then 
+      return fs.execute(vars.CHMOD, mode, pathname)
+   else
+      return false
+   end
 end
 
 --- Apply a patch.
 -- @param patchname string: The filename of the patch.
 function apply_patch(patchname)
-   return fs.execute("patch -p1 -f -i ", patchname)
+   return fs.execute(vars.PATCH.." -p1 -f -i ", patchname)
 end
 
 --- Unpack an archive.
@@ -264,12 +278,12 @@ function unpack_archive(archive)
    local ok
    if archive:match("%.tar%.gz$") or archive:match("%.tgz$") then
       -- ok = fs.execute("tar zxvpf ", archive)
-         ok = fs.execute_string("gunzip -c "..archive.."|tar -xf -")
+         ok = fs.execute_string(vars.GUNZIP.." -c "..archive.."|"..vars.TAR.." -xf -")
    elseif archive:match("%.tar%.bz2$") then
       -- ok = fs.execute("tar jxvpf ", archive)
-         ok = fs.execute_string("bunzip2 -c "..archive.."|tar -xf -")
+         ok = fs.execute_string(vars.BUNZIP2.." -c "..archive.."|tar -xf -")
    elseif archive:match("%.zip$") then
-      ok = fs.execute("unzip ", archive)
+      ok = fs.execute(vars.UNZIP, archive)
    elseif archive:match("%.lua$") or archive:match("%.c$") then
       -- Ignore .lua and .c files; they don't need to be extracted.
       return true
@@ -283,63 +297,28 @@ function unpack_archive(archive)
    return true
 end
 
+local md5_cmd = {
+   md5sum = vars.MD5SUM,
+   openssl = vars.OPENSSL.." md5",
+   md5 = vars.MD5,
+}
+
 --- Get the MD5 checksum for a file.
 -- @param file string: The file to be computed.
 -- @return string: The MD5 checksum
-function get_md5(file, md5sum)
-   file = fs.absolute_name(file)
-   local computed
-   if cfg.md5checker == "md5sum" then
-      local pipe = io.popen("md5sum "..file)
-      computed = pipe:read("*l")
-      pipe:close()
-      if computed then
-         computed = computed:gsub("[^%x]+", ""):sub(1,32)
-      end
-   elseif cfg.md5checker == "openssl" then
-      local pipe = io.popen("openssl md5 "..file)
-      computed = pipe:read("*l")
-      pipe:close()
-      if computed then
-         computed = computed:sub(-32)
-      end
-   elseif cfg.md5checker == "md5" then
-      local pipe = io.popen("md5 "..file)
-      computed = pipe:read("*l")
-      pipe:close()
-      if computed then
-         computed = computed:sub(-32)
-      end
-   end
-   return computed
+function get_md5(file)
+   local cmd = md5_cmd[cfg.md5checker]
+   if not cmd then return nil end
+   local pipe = io.popen(cmd.." "..fs.absolute_name(file))
+   local computed = pipe:read("*a")
+   pipe:close()
+   if not computed then return nil end
+   return computed:match("("..("%x"):rep(32)..")")
 end
 
---- Unpack an archive.
--- Extract the contents of an archive, detecting its format by
--- filename extension.
--- @param archive string: Filename of archive.
--- @return boolean or (boolean, string): true on success, false and an error message on failure.
-function unpack_archive(archive)
-   assert(type(archive) == "string")
-
-   local ok
-   if archive:match("%.tar%.gz$") or archive:match("%.tgz$") then
-      -- ok = fs.execute("tar zxvpf ", archive)
-      ok = fs.execute_string("gunzip -c "..archive.."|tar -xf -")
-   elseif archive:match("%.tar%.bz2$") then
-      -- ok = fs.execute("tar jxvpf ", archive)
-      ok = fs.execute_string("bunzip2 -c "..archive.."|tar -xf -")
-   elseif archive:match("%.zip$") then
-      ok = fs.execute("unzip ", archive)
-   elseif archive:match("%.lua$") or archive:match("%.c$") then
-      -- Ignore .lua and .c files; they don't need to be extracted.
-      return true
-   else
-      local ext = archive:match(".*(%..*)")
-      return false, "Unrecognized filename extension "..(ext or "")
-   end
-   if not ok then
-      return false, "Failed extracting "..archive
-   end
-   return true
+function get_permissions(filename)
+   local pipe = io.popen(vars.STAT.." "..vars.STATFLAG.." "..fs.Q(filename))
+   local ret = pipe:read("*l")
+   pipe:close()
+   return ret
 end
