@@ -1,4 +1,5 @@
 
+--- Module which builds the index.html page to be used in rocks servers.
 module("luarocks.index", package.seeall)
 
 local util = require("luarocks.util")
@@ -64,6 +65,7 @@ local index_package_start = [[
 <td class="package">
 <p><a name="$anchor"></a><b>$package</b> - $summary<br/>
 </p><blockquote><p>$detailed<br/>
+$externaldependencies
 <font size="-1"><a href="$original">latest sources</a> $homepage | License: $license</font></p>
 </blockquote></a></td>
 <td class="version">
@@ -83,6 +85,34 @@ local index_footer = [[
 </html>
 ]]
 
+function format_external_dependencies(rockspec)
+   if rockspec.external_dependencies then
+      local deplist = {}
+      local listed_set = {}
+      local plats = nil
+      for name, desc in util.sortedpairs(rockspec.external_dependencies) do
+         if name ~= "platforms" then
+            table.insert(deplist, name:lower())
+            listed_set[name] = true
+         else
+            plats = desc
+         end
+      end
+      if plats then
+         for plat, entries in util.sortedpairs(plats) do
+            for name, desc in util.sortedpairs(entries) do
+               if not listed_set[name] then
+                  table.insert(deplist, name:lower() .. " (on "..plat..")")
+               end
+            end
+         end
+      end
+      return '<p><b>External dependencies:</b> ' .. table.concat(deplist, ',&nbsp;').. '</p>'
+   else
+      return ""
+   end
+end
+
 function make_index(repo)
    if not fs.is_dir(repo) then
       return nil, "Cannot access repository at "..repo
@@ -95,35 +125,35 @@ function make_index(repo)
       local latest_rockspec = nil
       local output = index_package_start
       for version, data in util.sortedpairs(version_list, deps.compare_versions) do
-         local out_versions = {}
-         local arches = 0
-         output = output..version
-         local sep = ':&nbsp;'
+         local versions = {}
+         output = output..version..':&nbsp;'
+         table.sort(data, function(a,b) return a.arch < b.arch end)
          for _, item in ipairs(data) do
-            output = output .. sep .. '<a href="$url">'..item.arch..'</a>'
-            sep = ',&nbsp;'
+            local link = '<a href="$url">'..item.arch..'</a>'
             if item.arch == 'rockspec' then
                local rs = ("%s-%s.rockspec"):format(package, version)
                if not latest_rockspec then latest_rockspec = rs end
-               output = output:gsub("$url", rs)
+               link = link:gsub("$url", rs)
             else
-               output = output:gsub("$url", ("%s-%s.%s.rock"):format(package, version, item.arch))
+               link = link:gsub("$url", ("%s-%s.%s.rock"):format(package, version, item.arch))
             end
+            table.insert(versions, link)
          end
-         output = output .. '<br/>'
-         output = output:gsub("$na", arches)
+         output = output .. table.concat(versions, ',&nbsp;') .. '<br/>'
       end
       output = output .. index_package_end
       if latest_rockspec then
          local rockspec = persist.load_into_table(dir.path(repo, latest_rockspec))
+         local descript = rockspec.description or {}
          local vars = {
             anchor = package,
             package = rockspec.package,
             original = rockspec.source.url,
-            summary = rockspec.description.summary or "",
-            detailed = rockspec.description.detailed or "",
-            license = rockspec.description.license or "N/A",
-            homepage = rockspec.description.homepage and ("| <a href="..rockspec.description.homepage..">project homepage</a>") or ""
+            summary = descript.summary or "",
+            detailed = descript.detailed or "",
+            license = descript.license or "N/A",
+            homepage = descript.homepage and ("| <a href="..descript.homepage..">project homepage</a>") or "",
+            externaldependencies = format_external_dependencies(rockspec)
          }
          vars.detailed = vars.detailed:gsub("\n\n", "</p><p>"):gsub("%s+", " ")
          output = output:gsub("$(%w+)", vars)
