@@ -8,9 +8,12 @@ module("luarocks.make", package.seeall)
 local build = require("luarocks.build")
 local fs = require("luarocks.fs")
 local util = require("luarocks.util")
+local cfg = require("luarocks.cfg")
+local fetch = require("luarocks.fetch")
+local pack = require("luarocks.pack")
 
 help_summary = "Compile package in current directory using a rockspec."
-help_arguments = "[<rockspec>]"
+help_arguments = "[--pack-binary-rock] [<rockspec>]"
 help = [[
 Builds sources in the current directory, but unlike "build",
 it does not fetch sources, etc., assuming everything is 
@@ -21,6 +24,10 @@ is found, you must specify which to use, through the command-line.
 This command is useful as a tool for debugging rockspecs. 
 To install rocks, you'll normally want to use the "install" and
 "build" commands. See the help on those for details.
+
+If --pack-binary-rock is passed, the rock is not installed;
+instead, a .rock file with the contents of compilation is produced
+in the current directory.
 ]]
 
 --- Driver function for "make" command.
@@ -50,5 +57,15 @@ function run(...)
       return nil, "Invalid argument: 'make' takes a rockspec as a parameter. See help."
    end
 
-   return build.build_rockspec(rockspec, false, true)
+   if flags["pack-binary-rock"] then
+      local rspec, err, errcode = fetch.load_rockspec(rockspec)
+      if not rspec then
+         return nil, err
+      end
+      return pack.pack_binary_rock(rspec.name, rspec.version, build.build_rockspec, rockspec, false, true)
+   else
+      local ok, err = fs.check_command_permissions(flags)
+      if not ok then return nil, err end
+      return build.build_rockspec(rockspec, false, true)
+   end
 end
