@@ -138,10 +138,7 @@ function disk_search(repo, query, results)
    
    for _, name in pairs(fs.list_dir(repo)) do
       local pathname = dir.path(repo, name)
-      local rname, rversion, rarch = path.parse_rock_name(name)
-      if not rname then
-         rname, rversion, rarch = path.parse_rockspec_name(name)
-      end
+      local rname, rversion, rarch = path.parse_name(name)
       if fs.is_dir(pathname) then
          for _, version in pairs(fs.list_dir(pathname)) do
             if version:match("-%d+$") then
@@ -269,11 +266,11 @@ function find_suitable_rock(query)
    if not results then
       return nil, err
    end
-   local first = results and next(results)
-   if first and next(results, first) == nil then
-      return pick_latest_version(query.name, results[first])
-   elseif not first then
+   local first = next(results)
+   if not first then
       return nil, "No results matching query were found."
+   elseif not next(results, first) then
+      return pick_latest_version(query.name, results[first])
    else
       return results
    end
@@ -283,25 +280,26 @@ end
 -- @param results table: A table where keys are package names and versions
 -- are tables matching version strings to an array of rocks servers.
 -- @param show_repo boolean or nil: Whether to show repository
+-- @param long boolean or nil: Whether to show module files
 -- information or not. Default is true.
-function print_results(results, show_repo)
+function print_results(results, show_repo, long)
    assert(type(results) == "table")
    assert(type(show_repo) == "boolean" or not show_repo)
    -- Force display of repo location for the time being
    show_repo = true -- show_repo == nil and true or show_repo
    
    for package, versions in util.sortedpairs(results) do
-      print(package)
+      util.printout(package)
       for version, repos in util.sortedpairs(versions, deps.compare_versions) do
          if show_repo then
             for _, repo in ipairs(repos) do
-               print("   "..version.." ("..repo.arch..") - "..repo.repo)
+               util.printout("   "..version.." ("..repo.arch..") - "..repo.repo)
             end
          else
-            print("   "..version)
+            util.printout("   "..version)
          end
       end
-      print()
+      util.printout()
    end
 end
 
@@ -333,8 +331,8 @@ end
 -- user possibilities if it couldn't narrow down a single match.
 -- @param action function: A function that takes a .src.rock or
 -- .rockspec URL as a parameter.
--- @string name string: A rock name
--- @string version string or nil: A version number may also be given.
+-- @param name string: A rock name
+-- @param version string or nil: A version number may also be given.
 -- @return The result of the action function, or nil and an error message. 
 function act_on_src_or_rockspec(action, name, version)
    assert(type(action) == "function")
@@ -347,10 +345,10 @@ function act_on_src_or_rockspec(action, name, version)
    if type(results) == "string" then
       return action(results)
    elseif type(results) == "table" and next(results) then
-      print("Multiple search results were returned.")
-      print()
-      print("Search results:")
-      print("---------------")
+      util.printout("Multiple search results were returned.")
+      util.printout()
+      util.printout("Search results:")
+      util.printout("---------------")
       print_results(results)
       return nil, "Please narrow your query."
    else
@@ -374,27 +372,27 @@ function run(...)
       return nil, "Enter name and version or use --all; see help."
    end
    
-   local query = make_query(name, version)
+   local query = make_query(name:lower(), version)
    query.exact_name = false
    local results, err = search_repos(query)
    if not results then
       return nil, err
    end
-   print()
-   print("Search results:")
-   print("===============")
-   print()
+   util.printout()
+   util.printout("Search results:")
+   util.printout("===============")
+   util.printout()
    local sources, binaries = split_source_and_binary_results(results)
    if next(sources) and not flags["binary"] then
-      print("Rockspecs and source rocks:")
-      print("---------------------------")
-      print()
+      util.printout("Rockspecs and source rocks:")
+      util.printout("---------------------------")
+      util.printout()
       print_results(sources, true)
    end
    if next(binaries) and not flags["source"] then    
-      print("Binary and pure-Lua rocks:")
-      print("--------------------------")
-      print()
+      util.printout("Binary and pure-Lua rocks:")
+      util.printout("--------------------------")
+      util.printout()
       print_results(binaries, true)
    end
    return true
