@@ -45,6 +45,8 @@ end
 --- Load a .rock file to the given directory and unpack it inside it.
 -- @param rock_file string: The URL for a .rock file.
 -- @param dir_name string: The directory where to unpack.
+-- @param kind string: the kind of rock file, as in the second-level
+-- extension in the rock filename (eg. "src", "all", "linux-x86")
 -- @return table or (nil, string): the loaded rockspec table or 
 -- nil and an error message.
 local function unpack_rock(rock_file, dir_name, kind)
@@ -116,10 +118,10 @@ local function run_unpacker(file)
             return nil, "Failed copying unpacked rockspec into unpacked source directory."
          end
       end
-      print()   
-      print("Done. You may now enter directory ")
-      print(dir.path(dir_name, rockspec.source.dir))
-      print("and type 'luarocks make' to build.")
+      util.printout()   
+      util.printout("Done. You may now enter directory ")
+      util.printout(dir.path(dir_name, rockspec.source.dir))
+      util.printout("and type 'luarocks make' to build.")
    end
    util.remove_scheduled_function(rollback)
    return true
