@@ -1,10 +1,11 @@
 
-local global_env = _G
-
---- Utility functions shared by other modules.
+--- Assorted utilities for managing tables, plus a scheduler for rollback functions.
 -- Does not requires modules directly (only as locals
 -- inside specific functions) to avoid interdependencies,
 -- as this is used in the bootstrapping stage of luarocks.cfg.
+
+local global_env = _G
+
 module("luarocks.util", package.seeall)
 
 local scheduled_functions = {}
@@ -26,7 +27,7 @@ end
 
 --- Unschedule a function.
 -- This is useful for cancelling a rollback of a completed operation.
--- @param table: The token representing the scheduled function that was
+-- @param item table: The token representing the scheduled function that was
 -- returned from the schedule_function call.
 function remove_scheduled_function(item)
    for k, v in pairs(scheduled_functions) do
@@ -84,7 +85,11 @@ function deep_merge(dst, src)
          if not dst[k] then
             dst[k] = {}
          end
-         deep_merge(dst[k], v)
+         if type(dst[k]) == "table" then
+            deep_merge(dst[k], v)
+         else
+            dst[k] = v
+         end
       else
          dst[k] = v
       end
@@ -124,12 +129,6 @@ end
 
 local var_format_pattern = "%$%((%a[%a%d_]+)%)"
 
---- Display a warning message.
--- @param msg string: the warning message
-function warning(msg)
-   print("Warning: "..msg)
-end
-
 --- Create a new shallow copy of a table: a new table with
 -- the same keys and values. Keys point to the same objects as
 -- the original table (ie, does not copy recursively).
@@ -144,7 +143,7 @@ local function make_shallow_copy(tbl)
 end
 
 -- Check if a set of needed variables are referenced
--- somewher in a list of definitions, warning the user
+-- somewhere in a list of definitions, warning the user
 -- about any unused ones. Each key in needed_set should
 -- appear as a $(XYZ) variable at least once as a
 -- substring of some value of var_defs.
@@ -216,6 +215,20 @@ function keys(tbl)
    return ks
 end
 
+local function default_sort(a, b)
+   local ta = type(a)
+   local tb = type(b)
+   if ta == "number" and tb == "number" then
+      return a < b
+   elseif ta == "number" then
+      return true
+   elseif tb == "number" then
+      return false
+   else
+      return tostring(a) < tostring(b)
+   end
+end
+
 -- The iterator function used internally by util.sortedpairs.
 -- @param tbl table: The table to be iterated.
 -- @param sort_function function or nil: An optional comparison function
@@ -223,7 +236,7 @@ end
 -- @see sortedpairs
 local function sortedpairs_iterator(tbl, sort_function)
    local ks = keys(tbl)
-   table.sort(ks, sort_function)
+   table.sort(ks, sort_function or default_sort)
    for _, k in ipairs(ks) do
       coroutine.yield(k, tbl[k])
    end
@@ -243,6 +256,51 @@ function starts_with(s, prefix)
    return s:sub(1,#prefix) == prefix
 end
 
+--- Print a line to standard output
+function printout(...)
+   io.stdout:write(table.concat({...},"\t"))
+   io.stdout:write("\n")
+end
+
+--- Print a line to standard error
+function printerr(...)
+   io.stderr:write(table.concat({...},"\t"))
+   io.stderr:write("\n")
+end
+
+--- Display a warning message.
+-- @param msg string: the warning message
+function warning(msg)
+   printerr("Warning: "..msg)
+end
+
+-- from http://lua-users.org/wiki/SplitJoin
+-- by PhilippeLhoste
+function split_string(str, delim, maxNb)
+   -- Eliminate bad cases...
+   if string.find(str, delim) == nil then
+      return { str }
+   end
+   if maxNb == nil or maxNb < 1 then
+      maxNb = 0    -- No limit
+   end
+   local result = {}
+   local pat = "(.-)" .. delim .. "()"
+   local nb = 0
+   local lastPos
+   for part, pos in string.gfind(str, pat) do
+      nb = nb + 1
+      result[nb] = part
+      lastPos = pos
+      if nb == maxNb then break end
+   end
+   -- Handle the last field
+   if nb ~= maxNb then
+      result[nb + 1] = string.sub(str, lastPos)
+   end
+   return result
+end
+
 --[[
 Author: Julio Manuel Fernandez-Diaz
 Date:   January 12, 2007
@@ -261,7 +319,7 @@ which logically are exactly not equivalent to the original code.
 This routine can serve for pretty formating tables with
 proper indentations, apart from printing them:
 
-print(table.show(t, "t"))   -- a typical use
+io.write(table.show(t, "t"))   -- a typical use
 
 Heavily based on "Saving tables with cycles", PIL2, p. 113.
 
@@ -335,4 +393,3 @@ function show_table(t, name, indent)
    addtocart(t, name, indent)
    return cart .. autoref
 end
-
